'*** Declare Variables for Driver Function Locations
GLOBAL hLib AS LONG                        'Driver
GLOBAL hShowTerminal AS LONG               'ShowTerminal
GLOBAL hHideTerminal AS LONG               'HideTerminal
GLOBAL hPortOpen AS LONG                   'Port Open
GLOBAL hPortClose AS LONG                  'Port Close
GLOBAL hIsPortOpen AS LONG                 'Is Port Open
GLOBAL hSendToPort AS LONG                 'Send to Port
GLOBAL hReadFromPort AS LONG               'Read from Port
GLOBAL hCountChars AS LONG                 'Count Chars at Port
GLOBAL hSearchForChars AS LONG             'Search for Chars
GLOBAL hClearPort AS LONG                  'Clear Port
GLOBAL hRemoveFromPort AS LONG             'Remove Chars from Port
GLOBAL hGetMotorPosition AS LONG           'Get Motor Position
GLOBAL hWaitForChar AS LONG                'Wait for Char
GLOBAL hWaitForCharMotorPosition AS LONG   'Wait for Char and Get Motor Position while waiting
GLOBAL hDisableDriverFunctions AS LONG     'Disable Driver Functions for recovery
GLOBAL hEnableDriverFunctions AS LONG     'Disable Driver Functions for recovery

'*** Declare wrapper functions
DECLARE FUNCTION LoadDriver(PathToDll AS ASCIIZ)AS LONG
DECLARE FUNCTION ReleaseDriver()AS LONG
DECLARE FUNCTION DriverTerminalShowState(BYVAL StateToShow AS LONG, OPTIONAL BYVAL ParentHwnd AS LONG) AS LONG
DECLARE FUNCTION ShowTerminalSimple(BYVAL hParent&) AS LONG
DECLARE FUNCTION HideTerminalSimple() AS LONG
DECLARE FUNCTION PortOpen(BYVAL PortNumber AS LONG, BYVAL BaudRate AS LONG) AS LONG
DECLARE FUNCTION PortClose() AS LONG
DECLARE FUNCTION PortIsOpen() AS LONG
DECLARE FUNCTION PortSendCommands(CommandToSend AS ASCIIZ) AS LONG
DECLARE FUNCTION PortReadReply() AS STRING
DECLARE FUNCTION PortCountChars() AS LONG
DECLARE FUNCTION PortSearchForChars(CharsToFind AS ASCIIZ) AS LONG
DECLARE FUNCTION PortClear() AS LONG
DECLARE FUNCTION PortRemoveChars(StringToRemove AS ASCIIZ) AS LONG
DECLARE FUNCTION MotorPosition(BYVAL MotorNumber AS LONG) AS STRING
DECLARE FUNCTION PortWaitForChar(CharToWaitFor AS ASCIIZ, OPTIONAL BYVAL TimeOutTime AS LONG) AS LONG
DECLARE FUNCTION PortWaitForCharWithMotorPosition(CharToWaitFor AS ASCIIZ, BYVAL MotorNumber AS LONG, OPTIONAL BYVAL ReportToWindowHwnd AS LONG, OPTIONAL BYVAL TimeOutTime AS LONG) AS LONG
DECLARE SUB DriverResetFunctions()

'*** Wrapper functions
FUNCTION LoadDriver(PathToDll AS ASCIIZ)AS LONG
    hLib = LoadLibrary(PathToDll)
    SELECT CASE hLib
        CASE %False         'No driver found
        CASE ELSE          'Driver found so get memory locations of functions in the driver
            hShowTerminal = GetProcAddress(hLib, "ShowTerminalSimple")                                'ShowTerminal
            hHideTerminal = GetProcAddress(hLib, "HideTerminalSimple")                                'HideTerminal
            hPortOpen = GetProcAddress(hLib, "OpenPort")                                        'Port Open
            hPortClose = GetProcAddress(hLib, "ClosePort")                                      'Port Close
            hIsPortOpen = GetProcAddress(hLib, "IsPortOpen")                                    'Is Port Open
            hSendToPort = GetProcAddress(hLib, "DriverSendToPort")                              'Send to Port
            hReadFromPort = GetProcAddress(hLib, "ReadFromPort")                                'Read from Port
            hCountChars = GetProcAddress(hLib, "CountCharsAtPort")                              'Count Chars at Port
            hSearchForChars = GetProcAddress(hLib, "SearchForChars")                            'Search for Chars
            hClearPort = GetProcAddress(hLib, "ClearPort")                                      'Clear Port
            hRemoveFromPort = GetProcAddress(hLib, "RemoveFromPort")                            'Remove Chars from Port
            hGetMotorPosition = GetProcAddress(hLib, "GetMotorPosition")                        'Get Motor Position
            hWaitForChar = GetProcAddress(hLib, "WaitForChar")                                  'Wait for Char
            hWaitForCharMotorPosition = GetProcAddress(hLib, "WaitForCharWithMotorPosition")    'Wait for Char and Get Motor Position while waiting
            hDisableDriverFunctions = GetProcAddress(hLib, "DisableDriverFunctions")                'Reset Driver Functions for recovery
            hEnableDriverFunctions = GetProcAddress(hLib, "EnableDriverFunctions")                'Reset Driver Functions for recovery
    END SELECT
    FUNCTION = hLib
END FUNCTION

FUNCTION ReleaseDriver()AS LONG
    DIM lResult AS LONG
    SELECT CASE hLib
        CASE %False     'No driver to release
            FUNCTION = %False
        CASE ELSE       'Driver exists so release it
            CALL DWORD hIsPortOpen USING PortIsOpen TO lResult
            SELECT CASE lResult
                CASE %False     'Port is not open
                CASE %True      'Port is open so close it so other programs or calls do not fail
                    CALL DWORD hPortClose USING PortClose TO lResult
            END SELECT
            FreeLibrary hLib    'Free the handle to the driver
            FUNCTION = %True
    END SELECT
END FUNCTION

FUNCTION DriverTerminalShowState(BYVAL StateToShow AS LONG, OPTIONAL BYVAL ParentHwnd AS LONG) AS LONG
    DIM lResult AS LONG
    SELECT CASE StateToShow
        CASE %False     'Hide the terminal
            SELECT CASE hHideTerminal
                CASE %False         'No handle so can't do anything
                CASE ELSE
                    HideTerminalSimple
            END SELECT
        CASE %True      'Show the terminal
            SELECT CASE hShowTerminal
                CASE %False         'No handle so can't do anything
                CASE ELSE
                    ShowTerminalSimple(BYVAL ParentHwnd)
            END SELECT
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION ShowTerminalSimple(BYVAL hParent&) AS LONG
    DIM lResult AS LONG
    CALL DWORD hShowTerminal USING ShowTerminalSimple(BYVAL hParent&) TO lResult
END FUNCTION

FUNCTION HideTerminalSimple() AS LONG
    DIM lResult AS LONG
    CALL DWORD hHideTerminal
END FUNCTION

FUNCTION PortOpen(BYVAL PortNumber AS LONG, BYVAL BaudRate AS LONG) AS LONG
    DIM lResult AS LONG
    SELECT CASE hPortOpen
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hPortOpen USING PortOpen(BYVAL PortNumber, BYVAL BaudRate) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortClose() AS LONG
    DIM lResult AS LONG
    SELECT CASE hPortClose
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hPortClose USING PortClose TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortIsOpen() AS LONG
    DIM lResult AS LONG
    SELECT CASE hIsPortOpen
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hIsPortOpen USING PortIsOpen TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortSendCommands(CommandToSend AS ASCIIZ) AS LONG
    DIM lResult AS LONG
    SELECT CASE hSendToPort
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hSendToPort USING PortSendCommands(CommandToSend) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortReadReply() AS STRING
    DIM sResult AS STRING
    SELECT CASE hReadFromPort
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hReadFromPort USING PortReadReply() TO sResult
    END SELECT
    FUNCTION = sResult
END FUNCTION

FUNCTION PortCountChars() AS LONG
    DIM lResult AS LONG
    SELECT CASE hCountChars
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hCountChars USING PortCountChars() TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortSearchForChars(CharsToFind AS ASCIIZ) AS LONG
    DIM lResult AS LONG
    SELECT CASE hSearchForChars
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hSearchForChars USING PortSearchForChars(CharsToFind) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortClear() AS LONG
    DIM lResult AS LONG
    SELECT CASE hClearPort
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hClearPort USING PortClear() TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortRemoveChars(StringToRemove AS ASCIIZ) AS LONG
    DIM lResult AS LONG
    SELECT CASE hRemoveFromPort
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hRemoveFromPort USING PortRemoveChars(StringToRemove) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION MotorPosition(BYVAL MotorNumber AS LONG) AS STRING
    DIM sResult AS STRING
    SELECT CASE hGetMotorPosition
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hGetMotorPosition USING MotorPosition(BYVAL MotorNumber) TO sResult
    END SELECT
    FUNCTION = sResult
END FUNCTION

FUNCTION PortWaitForChar(CharToWaitFor AS ASCIIZ, OPTIONAL BYVAL TimeOutTime AS LONG) AS LONG
    DIM lResult AS LONG
    SELECT CASE hWaitForChar
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hWaitForChar USING PortWaitForChar(CharToWaitFor, BYVAL TimeOutTime) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

FUNCTION PortWaitForCharWithMotorPosition(CharToWaitFor AS ASCIIZ, BYVAL MotorNumber AS LONG, OPTIONAL BYVAL ReportToWindowHwnd AS LONG, OPTIONAL BYVAL TimeOutTime AS LONG) AS LONG
    DIM lResult AS LONG
    SELECT CASE hWaitForCharMotorPosition
        CASE %False     'No handle so can't do anything
        CASE ELSE
            CALL DWORD hWaitForCharMotorPosition  USING PortWaitForCharWithMotorPosition(CharToWaitFor, BYVAL MotorNumber, BYVAL ReportToWindowHwnd, BYVAL TimeOutTime) TO lResult
    END SELECT
    FUNCTION = lResult
END FUNCTION

SUB DriverResetFunctions()
    SELECT CASE hDisableDriverFunctions
        CASE %False     'No handle so can't do anything
        CASE ELSE
            SELECT CASE hEnableDriverFunctions
                CASE %False 'No handle so don't do anything if you can't re-enable the functions
                CASE ELSE
                    CALL DWORD hDisableDriverFunctions USING DriverResetFunctions
                    SLEEP 250   'Give functions a chance to catch on
                    CALL DWORD hEnableDriverFunctions USING DriverResetFunctions
            END SELECT
    END SELECT
END SUB
